@@ -64,7 +64,7 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
   verbose && enter(verbose, "Resampling TCN segments");
   ids <- unique(segs[["tcnId"]]);
-  for (ii in seq(along=ids)) {
+  for (ii in seq_along(ids)) {
     id <- ids[ii];
     verbose && enter(verbose, sprintf("TCN segment #%d of %d", ii, length(ids)));
 
@@ -77,7 +77,7 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsII$tcnStart[1];
     stop <- segsII$tcnEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     if (length(units) > 1) {
       # Resample
@@ -96,14 +96,14 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   # Resample DH segments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  for (jj in seq(length=nbrOfSegments)) {
+  for (jj in seq_len(nbrOfSegments)) {
     verbose && enter(verbose, sprintf("DH segment #%d of %d", jj, nbrOfSegments));
     segsJJ <- segs[jj,,drop=FALSE];
 
     # Identify loci in segment
     start <- segsJJ$dhStart[1];
     stop <- segsJJ$dhEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     # Identify SNPs and non-SNPs
     muN <- data$muN[units];
@@ -111,7 +111,7 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
     # Identify heterozygous SNPs
     isHet <- (muN == 1/2);
-    hets <- whichVector(isSnp & isHet);
+    hets <- which(isSnp & isHet);
 
     units <- units[hets];
 
@@ -136,7 +136,7 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
   verbose && enter(verbose, "Updating TCN fields");
   ids <- unique(segs[["tcnId"]]);
-  for (kk in seq(along=ids)) {
+  for (kk in seq_along(ids)) {
     id <- ids[kk];
     verbose && enter(verbose, sprintf("TCN segment #%d of %d", kk, length(ids)));
 
@@ -149,7 +149,7 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsKK$tcnStart[1];
     stop <- segsKK$tcnEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     # Update TCN mean level
     y <- data$CT[units];
@@ -161,7 +161,7 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
   verbose && exit(verbose);
 
   verbose && enter(verbose, "Updating DH fields");
-  for (kk in seq(length=nbrOfSegments)) {
+  for (kk in seq_len(nbrOfSegments)) {
     verbose && enter(verbose, sprintf("DH segment #%d of %d", kk, nbrOfSegments));
 
     segsKK <- segs[kk,,drop=FALSE];
@@ -169,7 +169,7 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsKK$dhStart[1];
     stop <- segsKK$dhEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     # Identify SNPs and non-SNPs
     muN <- data$muN[units];
@@ -177,7 +177,7 @@ setMethodS3("resampleC", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
     # Identify heterozygous SNPs
     isHet <- (muN == 1/2);
-    hets <- whichVector(isSnp & isHet);
+    hets <- which(isSnp & isHet);
 
     # Update DH mean level
     beta <- data[[by]][units];
@@ -231,7 +231,7 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
   verbose && enter(verbose, "Identifying nested TCN units and DH units");
   unitsList <- list();
   ids <- unique(segs[["tcnId"]]);
-  for (ii in seq(along=ids)) {
+  for (ii in seq_along(ids)) {
     id <- ids[ii];
     verbose && enter(verbose, sprintf("TCN segment #%d of %d", ii, length(ids)));
     unitsII <- list();
@@ -245,29 +245,29 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsII$tcnStart[1];
     stop <- segsII$tcnEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
     tcnUnits <- units;
 
     # For each DH segment in this TCN segment        
     dhList <- list();
-    for (jj in seq(length=n)) {
+    for (jj in seq_len(n)) {
       verbose && enter(verbose, sprintf("DH segment #%d of %d", jj, n));
       segsJJ <- segsII[jj,,drop=FALSE];
 
       # Identify loci in segment
       start <- segsJJ$dhStart[1];
       stop <- segsJJ$dhEnd[1];
-      units <- whichVector(start <= x & x <= stop);
+      units <- which(start <= x & x <= stop);
 
       # Identify SNPs and non-SNPs
       muN <- data$muN[units];
       isSnp <- is.finite(muN);
-      nonSnps <- whichVector(!isSnp);
+      nonSnps <- which(!isSnp);
 
       # Identify heterozygous SNPs
       isHet <- (muN == 1/2);
-      hets <- whichVector(isSnp &  isHet);
-      homs <- whichVector(isSnp & !isHet);
+      hets <- which(isSnp &  isHet);
+      homs <- which(isSnp & !isHet);
 
       dhList[[jj]] <- list(dhUnits=units, nonSnps=units[nonSnps], hets=units[hets], homs=units[homs]);
       verbose && exit(verbose);
@@ -298,7 +298,7 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   verbose && enter(verbose, "Resampling unit indices");
   unitsListS <- list();
-  for (ii in seq(along=unitsList)) {
+  for (ii in seq_along(unitsList)) {
     verbose && enter(verbose, sprintf("TCN segment #%d of %d", ii, length(unitsList)));
 
     unitsListII <- unitsList[[ii]];
@@ -353,14 +353,14 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
   units <- unlist(lapply(unitsList, FUN=function(x) x[[1]]), use.names=FALSE);
   unitsS <- unlist(lapply(unitsListS, FUN=function(x) x[[1]]), use.names=FALSE);
-  idxs <- seq(along=units);
+  idxs <- seq_along(units);
   idxs[units] <- unitsS;
 
   fields <- c("CT", "betaN", "betaT", "betaTN", "muN");
   fields <- intersect(names(data), fields);
   verbose && cat(verbose, "Fields:");
   verbose && print(verbose, fields);
-  for (ff in seq(along=fields)) {
+  for (ff in seq_along(fields)) {
     data[[ff]] <- data[[ff]][idxs];
   }
   data$units <- idxs;
@@ -374,7 +374,7 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
   verbose && enter(verbose, "Updating TCN fields");
   ids <- unique(segs[["tcnId"]]);
-  for (kk in seq(along=ids)) {
+  for (kk in seq_along(ids)) {
     id <- ids[kk];
     verbose && enter(verbose, sprintf("TCN segment #%d of %d", kk, length(ids)));
 
@@ -387,7 +387,7 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsKK$tcnStart[1];
     stop <- segsKK$tcnEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     # Update TCN mean level
     y <- data$CT[units];
@@ -399,7 +399,7 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
   verbose && exit(verbose);
 
   verbose && enter(verbose, "Updating DH fields");
-  for (kk in seq(length=nbrOfSegments)) {
+  for (kk in seq_len(nbrOfSegments)) {
     verbose && enter(verbose, sprintf("DH segment #%d of %d", kk, nbrOfSegments));
 
     segsKK <- segs[kk,,drop=FALSE];
@@ -407,7 +407,7 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsKK$dhStart[1];
     stop <- segsKK$dhEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     # Identify SNPs and non-SNPs
     muN <- data$muN[units];
@@ -415,7 +415,7 @@ setMethodS3("resampleA", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
     # Identify heterozygous SNPs
     isHet <- (muN == 1/2);
-    hets <- whichVector(isSnp & isHet);
+    hets <- which(isSnp & isHet);
 
     # Update DH mean level
     beta <- data[[by]][units];
@@ -472,7 +472,7 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
   verbose && enter(verbose, "Resampling TCN segments");
   ids <- unique(segs[["tcnId"]]);
-  for (ii in seq(along=ids)) {
+  for (ii in seq_along(ids)) {
     id <- ids[ii];
     verbose && enter(verbose, sprintf("TCN segment #%d of %d", ii, length(ids)));
 
@@ -485,14 +485,14 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsII$tcnStart[1];
     stop <- segsII$tcnEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     if (length(units) > 1) {
       # Resample
       unitsS <- resample(units, replace=TRUE);
   
       # Resample data
-      for (ff in seq(along=fields)) {
+      for (ff in seq_along(fields)) {
         data[[ff]][units] <- data[[ff]][unitsS];
       }
     }
@@ -504,14 +504,14 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   # Resample DH segments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-  for (jj in seq(length=nbrOfSegments)) {
+  for (jj in seq_len(nbrOfSegments)) {
     verbose && enter(verbose, sprintf("TCN segment #%d of %d", jj, nbrOfSegments));
     segsJJ <- segs[jj,,drop=FALSE];
 
     # Identify loci in segment
     start <- segsJJ$dhStart[1];
     stop <- segsJJ$dhEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     # Identify SNPs and non-SNPs
     muN <- data$muN[units];
@@ -519,7 +519,7 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
     # Identify heterozygous SNPs
     isHet <- (muN == 1/2);
-    hets <- whichVector(isSnp & isHet);
+    hets <- which(isSnp & isHet);
 
     units <- units[hets];
 
@@ -528,7 +528,7 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
       unitsS <- resample(units, replace=TRUE);
   
       # Resample data
-      for (ff in seq(along=fields)) {
+      for (ff in seq_along(fields)) {
         data[[ff]][units] <- data[[ff]][unitsS];
       }
     }
@@ -544,7 +544,7 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
   verbose && enter(verbose, "Updating TCN fields");
   ids <- unique(segs[["tcnId"]]);
-  for (kk in seq(along=ids)) {
+  for (kk in seq_along(ids)) {
     id <- ids[kk];
     verbose && enter(verbose, sprintf("TCN segment #%d of %d", kk, length(ids)));
 
@@ -557,7 +557,7 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsKK$tcnStart[1];
     stop <- segsKK$tcnEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     # Update TCN mean level
     y <- data$CT[units];
@@ -569,7 +569,7 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
   verbose && exit(verbose);
 
   verbose && enter(verbose, "Updating DH fields");
-  for (kk in seq(length=nbrOfSegments)) {
+  for (kk in seq_len(nbrOfSegments)) {
     verbose && enter(verbose, sprintf("DH segment #%d of %d", kk, nbrOfSegments));
 
     segsKK <- segs[kk,,drop=FALSE];
@@ -577,7 +577,7 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
     # Identify loci in segment
     start <- segsKK$dhStart[1];
     stop <- segsKK$dhEnd[1];
-    units <- whichVector(start <= x & x <= stop);
+    units <- which(start <= x & x <= stop);
 
     # Identify SNPs and non-SNPs
     muN <- data$muN[units];
@@ -585,7 +585,7 @@ setMethodS3("resampleB", "PairedPSCBS", function(fit, by=c("betaTN", "betaT"), .
 
     # Identify heterozygous SNPs
     isHet <- (muN == 1/2);
-    hets <- whichVector(isSnp & isHet);
+    hets <- which(isSnp & isHet);
 
     # Update DH mean level
     beta <- data[[by]][units];
